CANCELLED_APPT = "cancelled"
CLINIC = "clinic"
COMPLETE_APPT = "done"
FUTURE_DATE = "future_date"
INCOMPLETE_APPT = "incomplete"
IN_PROGRESS_APPT = "in_progress"
LAST_MONTH = "last_month"
LAST_WEEK = "last_week"
LATE_APPT = "late"
MISSED_APPT = "missed"
NEW_APPT = "new"
NEXT_MONTH = "next_month"
NEXT_WEEK = "next_week"
NOT_NULL = "not_null"
ONTIME_APPT = "ontime"
PAST_DATE = "past_date"
SCHEDULED_APPT = "scheduled"
THIS_WEEK = "this_week"
TODAY = "today"
TOMORROW = "tomorrow"
UNSCHEDULED_APPT = "unscheduled"
